/*
 * Copyright 2006, 2007 Odysseus Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.odysseus.el.tree.impl.ast;

import javax.el.ELContext;
import javax.el.ELException;

import de.odysseus.el.misc.TypeConversions;
import de.odysseus.el.tree.Bindings;

public final class AstChoice extends AstRightValue {
	private final AstNode question, yes, no;
	
	public AstChoice(AstNode question, AstNode yes, AstNode no) {
		this.question = question;
		this.yes = yes;
		this.no = no;
	}

	@Override 
	public Object eval(Bindings bindings, ELContext context) throws ELException {
		Boolean value = TypeConversions.coerceToBoolean(question.eval(bindings, context));
		return value.booleanValue() ? yes.eval(bindings, context) : no.eval(bindings, context);
	}

	@Override
	public String toString() {
		return "?";
	}	

	@Override 
	public void appendStructure(StringBuilder b, Bindings bindings) {
		question.appendStructure(b, bindings);
		b.append(" ? ");
		yes.appendStructure(b, bindings);
		b.append(" : ");
		no.appendStructure(b, bindings);
	}

	public int getCardinality() {
		return 3;
	}

	public AstNode getChild(int i) {
		return i == 0 ? question : i == 1 ? yes : i == 2 ? no : null;
	}
}
