/*
 * Copyright 2006, 2007 Odysseus Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package de.odysseus.el;

import javax.el.ELException;

public class ObjectValueExpressionTest extends TestCase {

	public void testHashCode() {
		assertEquals("foo".hashCode(), new ObjectValueExpression("foo", Object.class).hashCode());
	}

	public void testEqualsObject() {
		assertTrue(new ObjectValueExpression("foo", Object.class).equals(new ObjectValueExpression("foo", Object.class)));
		assertTrue(new ObjectValueExpression(new String("foo"), Object.class).equals(new ObjectValueExpression("foo", Object.class)));
		assertFalse(new ObjectValueExpression("foo", Object.class).equals(new ObjectValueExpression("bar", Object.class)));
	}

	public void testGetValue() {
		assertEquals("foo", new ObjectValueExpression("foo", Object.class).getValue(null));
	}

	public void testGetExpressionString() {
		assertNull(new ObjectValueExpression("foo", Object.class).getExpressionString());
	}

	public void testIsLiteralText() {
		assertFalse(new ObjectValueExpression("foo", Object.class).isLiteralText());
	}

	public void testGetType() {
		assertNull(new ObjectValueExpression("foo", Object.class).getType(null));
	}

	public void testIsReadOnly() {
		assertTrue(new ObjectValueExpression("foo", Object.class).isReadOnly(null));
	}

	public void testSetValue() {
		try {
			new ObjectValueExpression("foo", Object.class).setValue(null, "bar");
			fail();
		} catch (ELException e) {}
	}

	public void testSerialize() throws Exception {
		ObjectValueExpression expression = new ObjectValueExpression("foo", Object.class);
		assertEquals(expression, deserialize(serialize(expression)));
	}
}
