/*
 * Copyright 2006, 2007 Odysseus Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package de.odysseus.el.misc;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;

import junit.framework.TestCase;

public class BooleanOperationsTest extends TestCase {

	/**
	 * Test enum type
	 */
	static enum Foo { BAR, BAZ };

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.lt(Object, Object)'
	 */
	public void testLt() {
		assertFalse(BooleanOperations.lt(Boolean.TRUE, Boolean.TRUE));
		assertFalse(BooleanOperations.lt(null, Boolean.TRUE));
		assertFalse(BooleanOperations.lt(Boolean.TRUE, null));
		assertTrue(BooleanOperations.lt("1", new BigDecimal("2")));
		assertFalse(BooleanOperations.lt(new BigDecimal("1"), "1"));
		assertFalse(BooleanOperations.lt(new BigDecimal("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new Float("2")));
		assertFalse(BooleanOperations.lt(new Float("1"), "1"));
		assertFalse(BooleanOperations.lt(new Float("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new Double("2")));
		assertFalse(BooleanOperations.lt(new Double("1"), "1"));
		assertFalse(BooleanOperations.lt(new Double("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new BigInteger("2")));
		assertFalse(BooleanOperations.lt(new BigInteger("1"), "1"));
		assertFalse(BooleanOperations.lt(new BigInteger("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new Byte("2")));
		assertFalse(BooleanOperations.lt(new Byte("1"), "1"));
		assertFalse(BooleanOperations.lt(new Byte("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new Short("2")));
		assertFalse(BooleanOperations.lt(new Short("1"), "1"));
		assertFalse(BooleanOperations.lt(new Short("2"), "1"));
		assertTrue(BooleanOperations.lt(new Character('a'), new Character('b')));
		assertFalse(BooleanOperations.lt(new Character('a'), new Character('a')));
		assertFalse(BooleanOperations.lt(new Character('b'), new Character('a')));
		assertTrue(BooleanOperations.lt("1", new Integer("2")));
		assertFalse(BooleanOperations.lt(new Integer("1"), "1"));
		assertFalse(BooleanOperations.lt(new Integer("2"), "1"));
		assertTrue(BooleanOperations.lt("1", new Long("2")));
		assertFalse(BooleanOperations.lt(new Long("1"), "1"));
		assertFalse(BooleanOperations.lt(new Long("2"), "1"));
		assertTrue(BooleanOperations.lt("a", "b"));
		assertFalse(BooleanOperations.lt("a", "a"));
		assertFalse(BooleanOperations.lt("b", "a"));
		try {
			BooleanOperations.lt(getClass(), new Character('a'));
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.lt(new Character('a'), getClass());
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.lt(getClass(), new Long(0));
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.gt(Object, Object)'
	 */
	public void testGt() {
		assertFalse(BooleanOperations.gt(Boolean.TRUE, Boolean.TRUE));
		assertFalse(BooleanOperations.gt(null, Boolean.TRUE));
		assertFalse(BooleanOperations.gt(Boolean.TRUE, null));
		assertFalse(BooleanOperations.gt("1", new BigDecimal("2")));
		assertFalse(BooleanOperations.gt(new BigDecimal("1"), "1"));
		assertTrue(BooleanOperations.gt(new BigDecimal("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new Float("2")));
		assertFalse(BooleanOperations.gt(new Float("1"), "1"));
		assertTrue(BooleanOperations.gt(new Float("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new Double("2")));
		assertFalse(BooleanOperations.gt(new Double("1"), "1"));
		assertTrue(BooleanOperations.gt(new Double("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new BigInteger("2")));
		assertFalse(BooleanOperations.gt(new BigInteger("1"), "1"));
		assertTrue(BooleanOperations.gt(new BigInteger("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new Byte("2")));
		assertFalse(BooleanOperations.gt(new Byte("1"), "1"));
		assertTrue(BooleanOperations.gt(new Byte("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new Short("2")));
		assertFalse(BooleanOperations.gt(new Short("1"), "1"));
		assertTrue(BooleanOperations.gt(new Short("2"), "1"));
		assertFalse(BooleanOperations.gt(new Character('a'), new Character('b')));
		assertFalse(BooleanOperations.gt(new Character('a'), new Character('a')));
		assertTrue(BooleanOperations.gt(new Character('b'), new Character('a')));
		assertFalse(BooleanOperations.gt("1", new Integer("2")));
		assertFalse(BooleanOperations.gt(new Integer("1"), "1"));
		assertTrue(BooleanOperations.gt(new Integer("2"), "1"));
		assertFalse(BooleanOperations.gt("1", new Long("2")));
		assertFalse(BooleanOperations.gt(new Long("1"), "1"));
		assertTrue(BooleanOperations.gt(new Long("2"), "1"));
		assertFalse(BooleanOperations.gt("a", "b"));
		assertFalse(BooleanOperations.gt("a", "a"));
		assertTrue(BooleanOperations.gt("b", "a"));
		try {
			BooleanOperations.gt(getClass(), new Character('a'));
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.gt(new Character('a'), getClass());
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.gt(getClass(), new Long(0));
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.ge(Object, Object)'
	 */
	public void testGe() {
		assertTrue(BooleanOperations.ge(Boolean.TRUE, Boolean.TRUE));
		assertFalse(BooleanOperations.ge(null, Boolean.TRUE));
		assertFalse(BooleanOperations.ge(Boolean.TRUE, null));
		assertFalse(BooleanOperations.ge("1", new BigDecimal("2")));
		assertTrue(BooleanOperations.ge(new BigDecimal("1"), "1"));
		assertTrue(BooleanOperations.ge(new BigDecimal("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new Float("2")));
		assertTrue(BooleanOperations.ge(new Float("1"), "1"));
		assertTrue(BooleanOperations.ge(new Float("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new Double("2")));
		assertTrue(BooleanOperations.ge(new Double("1"), "1"));
		assertTrue(BooleanOperations.ge(new Double("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new BigInteger("2")));
		assertTrue(BooleanOperations.ge(new BigInteger("1"), "1"));
		assertTrue(BooleanOperations.ge(new BigInteger("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new Byte("2")));
		assertTrue(BooleanOperations.ge(new Byte("1"), "1"));
		assertTrue(BooleanOperations.ge(new Byte("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new Short("2")));
		assertTrue(BooleanOperations.ge(new Short("1"), "1"));
		assertTrue(BooleanOperations.ge(new Short("2"), "1"));
		assertFalse(BooleanOperations.ge(new Character('a'), new Character('b')));
		assertTrue(BooleanOperations.ge(new Character('a'), new Character('a')));
		assertTrue(BooleanOperations.ge(new Character('b'), new Character('a')));
		assertFalse(BooleanOperations.ge("1", new Integer("2")));
		assertTrue(BooleanOperations.ge(new Integer("1"), "1"));
		assertTrue(BooleanOperations.ge(new Integer("2"), "1"));
		assertFalse(BooleanOperations.ge("1", new Long("2")));
		assertTrue(BooleanOperations.ge(new Long("1"), "1"));
		assertTrue(BooleanOperations.ge(new Long("2"), "1"));
		assertFalse(BooleanOperations.ge("a", "b"));
		assertTrue(BooleanOperations.ge("a", "a"));
		assertTrue(BooleanOperations.ge("b", "a"));
		try {
			BooleanOperations.ge(getClass(), new Character('a'));
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.ge(new Character('a'), getClass());
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.ge(getClass(), new Long(0));
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.le(Object, Object)'
	 */
	public void testLe() {
		assertTrue(BooleanOperations.le(Boolean.TRUE, Boolean.TRUE));
		assertFalse(BooleanOperations.le(null, Boolean.TRUE));
		assertFalse(BooleanOperations.le(Boolean.TRUE, null));
		assertTrue(BooleanOperations.le("1", new BigDecimal("2")));
		assertTrue(BooleanOperations.le(new BigDecimal("1"), "1"));
		assertFalse(BooleanOperations.le(new BigDecimal("2"), "1"));
		assertTrue(BooleanOperations.le("1", new Float("2")));
		assertTrue(BooleanOperations.le(new Float("1"), "1"));
		assertFalse(BooleanOperations.le(new Float("2"), "1"));
		assertTrue(BooleanOperations.le("1", new Double("2")));
		assertTrue(BooleanOperations.le(new Double("1"), "1"));
		assertFalse(BooleanOperations.le(new Double("2"), "1"));
		assertTrue(BooleanOperations.le("1", new BigInteger("2")));
		assertTrue(BooleanOperations.le(new BigInteger("1"), "1"));
		assertFalse(BooleanOperations.le(new BigInteger("2"), "1"));
		assertTrue(BooleanOperations.le("1", new Byte("2")));
		assertTrue(BooleanOperations.le(new Byte("1"), "1"));
		assertFalse(BooleanOperations.le(new Byte("2"), "1"));
		assertTrue(BooleanOperations.le("1", new Short("2")));
		assertTrue(BooleanOperations.le(new Short("1"), "1"));
		assertFalse(BooleanOperations.le(new Short("2"), "1"));
		assertTrue(BooleanOperations.le(new Character('a'), new Character('b')));
		assertTrue(BooleanOperations.le(new Character('a'), new Character('a')));
		assertFalse(BooleanOperations.le(new Character('b'), new Character('a')));
		assertTrue(BooleanOperations.le("1", new Integer("2")));
		assertTrue(BooleanOperations.le(new Integer("1"), "1"));
		assertFalse(BooleanOperations.le(new Integer("2"), "1"));
		assertTrue(BooleanOperations.le("1", new Long("2")));
		assertTrue(BooleanOperations.le(new Long("1"), "1"));
		assertFalse(BooleanOperations.le(new Long("2"), "1"));
		assertTrue(BooleanOperations.le("a", "b"));
		assertTrue(BooleanOperations.le("a", "a"));
		assertFalse(BooleanOperations.le("b", "a"));
		try {
			BooleanOperations.le(getClass(), new Character('a'));
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.le(new Character('a'), getClass());
			fail();
		} catch (Exception e) {}
		try {
			BooleanOperations.le(getClass(), new Long(0));
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.eq(Object, Object)'
	 */
	public void testEq() {
		assertTrue(BooleanOperations.eq(Boolean.TRUE, Boolean.TRUE));
		assertFalse(BooleanOperations.eq(null, Boolean.TRUE));
		assertFalse(BooleanOperations.eq(Boolean.TRUE, null));
		assertFalse(BooleanOperations.eq("1", new BigDecimal("2")));
		assertTrue(BooleanOperations.eq(new BigDecimal("1"), "1"));
		assertFalse(BooleanOperations.eq(new BigDecimal("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new Float("2")));
		assertTrue(BooleanOperations.eq(new Float("1"), "1"));
		assertFalse(BooleanOperations.eq(new Float("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new Double("2")));
		assertTrue(BooleanOperations.eq(new Double("1"), "1"));
		assertFalse(BooleanOperations.eq(new Double("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new BigInteger("2")));
		assertTrue(BooleanOperations.eq(new BigInteger("1"), "1"));
		assertFalse(BooleanOperations.eq(new BigInteger("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new Byte("2")));
		assertTrue(BooleanOperations.eq(new Byte("1"), "1"));
		assertFalse(BooleanOperations.eq(new Byte("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new Short("2")));
		assertTrue(BooleanOperations.eq(new Short("1"), "1"));
		assertFalse(BooleanOperations.eq(new Short("2"), "1"));
		assertFalse(BooleanOperations.eq(new Character('a'), new Character('b')));
		assertTrue(BooleanOperations.eq(new Character('a'), new Character('a')));
		assertFalse(BooleanOperations.eq(new Character('b'), new Character('a')));
		assertFalse(BooleanOperations.eq("1", new Integer("2")));
		assertTrue(BooleanOperations.eq(new Integer("1"), "1"));
		assertFalse(BooleanOperations.eq(new Integer("2"), "1"));
		assertFalse(BooleanOperations.eq("1", new Long("2")));
		assertTrue(BooleanOperations.eq(new Long("1"), "1"));
		assertFalse(BooleanOperations.eq(new Long("2"), "1"));
		assertFalse(BooleanOperations.eq(new Boolean(false), new Boolean(true)));
		assertTrue(BooleanOperations.eq(new Boolean(true), new Boolean(true)));
		assertTrue(BooleanOperations.eq(new Boolean(false), new Boolean(false)));
		assertTrue(BooleanOperations.eq(Foo.BAR, "BAR"));
		assertTrue(BooleanOperations.eq("BAR", Foo.BAR));
		assertFalse(BooleanOperations.eq(Foo.BAR, "BAZ"));
		try {
			BooleanOperations.eq(Foo.BAR, "FOO"); // coercion fails
			fail();
		} catch (Exception e) {}
		assertFalse(BooleanOperations.eq("a", "b"));
		assertTrue(BooleanOperations.eq("a", "a"));
		assertFalse(BooleanOperations.eq("b", "a"));
		assertFalse(BooleanOperations.eq(getClass(), new Character('a')));
		assertFalse(BooleanOperations.eq(new Character('a'), getClass()));
		try {
			BooleanOperations.eq(getClass(), new Long(0)); // coercion fails
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.ne(Object, Object)'
	 */
	public void testNe() {
		assertFalse(BooleanOperations.ne(Boolean.TRUE, Boolean.TRUE));
		assertTrue(BooleanOperations.ne(null, Boolean.TRUE));
		assertTrue(BooleanOperations.ne(Boolean.TRUE, null));
		assertTrue(BooleanOperations.ne("1", new BigDecimal("2")));
		assertFalse(BooleanOperations.ne(new BigDecimal("1"), "1"));
		assertTrue(BooleanOperations.ne(new BigDecimal("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new Float("2")));
		assertFalse(BooleanOperations.ne(new Float("1"), "1"));
		assertTrue(BooleanOperations.ne(new Float("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new Double("2")));
		assertFalse(BooleanOperations.ne(new Double("1"), "1"));
		assertTrue(BooleanOperations.ne(new Double("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new BigInteger("2")));
		assertFalse(BooleanOperations.ne(new BigInteger("1"), "1"));
		assertTrue(BooleanOperations.ne(new BigInteger("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new Byte("2")));
		assertFalse(BooleanOperations.ne(new Byte("1"), "1"));
		assertTrue(BooleanOperations.ne(new Byte("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new Short("2")));
		assertFalse(BooleanOperations.ne(new Short("1"), "1"));
		assertTrue(BooleanOperations.ne(new Short("2"), "1"));
		assertTrue(BooleanOperations.ne(new Character('a'), new Character('b')));
		assertFalse(BooleanOperations.ne(new Character('a'), new Character('a')));
		assertTrue(BooleanOperations.ne(new Character('b'), new Character('a')));
		assertTrue(BooleanOperations.ne("1", new Integer("2")));
		assertFalse(BooleanOperations.ne(new Integer("1"), "1"));
		assertTrue(BooleanOperations.ne(new Integer("2"), "1"));
		assertTrue(BooleanOperations.ne("1", new Long("2")));
		assertFalse(BooleanOperations.ne(new Long("1"), "1"));
		assertTrue(BooleanOperations.ne(new Long("2"), "1"));
		assertTrue(BooleanOperations.ne(new Boolean(false), new Boolean(true)));
		assertFalse(BooleanOperations.ne(new Boolean(true), new Boolean(true)));
		assertFalse(BooleanOperations.ne(new Boolean(false), new Boolean(false)));
		assertFalse(BooleanOperations.ne(Foo.BAR, "BAR"));
		assertFalse(BooleanOperations.ne("BAR", Foo.BAR));
		assertTrue(BooleanOperations.ne(Foo.BAR, "BAZ"));
		try {
			BooleanOperations.ne(Foo.BAR, "FOO"); // coercion fails
			fail();
		} catch (Exception e) {}
		assertTrue(BooleanOperations.ne("a", "b"));
		assertFalse(BooleanOperations.ne("a", "a"));
		assertTrue(BooleanOperations.ne("b", "a"));
		assertTrue(BooleanOperations.ne(getClass(), new Character('a')));
		assertTrue(BooleanOperations.ne(new Character('a'), getClass()));
		try {
			BooleanOperations.ne(getClass(), new Long(0)); // coercion fails
			fail();
		} catch (Exception e) {}
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.and(Object, Object)'
	 */
	public void testAnd() {
		assertFalse(BooleanOperations.and(Boolean.FALSE, Boolean.FALSE));
		assertFalse(BooleanOperations.and(Boolean.FALSE, Boolean.TRUE));
		assertFalse(BooleanOperations.and(Boolean.TRUE, Boolean.FALSE));
		assertTrue(BooleanOperations.and(Boolean.TRUE, Boolean.TRUE));
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.or(Object, Object)'
	 */
	public void testOr() {
		assertFalse(BooleanOperations.or(Boolean.FALSE, Boolean.FALSE));
		assertTrue(BooleanOperations.or(Boolean.FALSE, Boolean.TRUE));
		assertTrue(BooleanOperations.or(Boolean.TRUE, Boolean.FALSE));
		assertTrue(BooleanOperations.or(Boolean.TRUE, Boolean.TRUE));
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.not(Object)'
	 */
	public void testNot() {
		assertTrue(BooleanOperations.not(Boolean.FALSE));
		assertFalse(BooleanOperations.not(Boolean.TRUE));
	}

	/*
	 * Test method for 'de.odysseus.el.lang.BooleanOperations.empty(Object)'
	 */
	public void testEmpty() {
		assertTrue(BooleanOperations.empty(null));
		assertTrue(BooleanOperations.empty(""));
		assertTrue(BooleanOperations.empty(new Object[0]));
		assertTrue(BooleanOperations.empty(new HashMap()));
		assertTrue(BooleanOperations.empty(new ArrayList()));
		assertFalse(BooleanOperations.empty("foo"));
	}
}
