/*
 * Copyright 2006, 2007 Odysseus Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package de.odysseus.el.tree.impl.ast;

import javax.el.ELException;

import de.odysseus.el.TestCase;

public class AstBinaryTest extends TestCase {
	AstBinary parseNode(String expression) {
		return (AstBinary)parse(expression).getRoot().getChild(0);
	}

	public void testEval() {
		assertEquals(6l, parseNode("${4+2}").eval(null, null));
		assertEquals(8l, parseNode("${4*2}").eval(null, null));
		assertEquals(2d, parseNode("${4/2}").eval(null, null));
		assertEquals(0l, parseNode("${4%2}").eval(null, null));

		assertEquals(false, parseNode("${true && false}").eval(null, null));
		
		assertEquals(true, parseNode("${true || false}").eval(null, null));

		assertEquals(true, parseNode("${1 == 1}").eval(null, null));
		assertEquals(false, parseNode("${1 == 2}").eval(null, null));
		assertEquals(false, parseNode("${2 == 1}").eval(null, null));

		assertEquals(false, parseNode("${1 != 1}").eval(null, null));
		assertEquals(true, parseNode("${1 != 2}").eval(null, null));
		assertEquals(false, parseNode("${2 == 1}").eval(null, null));

		assertEquals(false, parseNode("${1 < 1}").eval(null, null));
		assertEquals(true, parseNode("${1 < 2}").eval(null, null));
		assertEquals(false, parseNode("${2 < 1}").eval(null, null));

		assertEquals(false, parseNode("${1 > 1}").eval(null, null));
		assertEquals(false, parseNode("${1 > 2}").eval(null, null));
		assertEquals(true, parseNode("${2 > 1}").eval(null, null));

		assertEquals(true, parseNode("${1 <= 1}").eval(null, null));
		assertEquals(true, parseNode("${1 <= 2}").eval(null, null));
		assertEquals(false, parseNode("${2 <= 1}").eval(null, null));

		assertEquals(true, parseNode("${1 >= 1}").eval(null, null));
		assertEquals(false, parseNode("${1 >= 2}").eval(null, null));
		assertEquals(true, parseNode("${2 >= 1}").eval(null, null));
	}

	public void testAppendStructure() {
		StringBuilder s = null;
		s = new StringBuilder();
		parseNode("${1+1}").appendStructure(s, null);
		parseNode("${1*1}").appendStructure(s, null);
		parseNode("${1/1}").appendStructure(s, null);
		parseNode("${1%1}").appendStructure(s, null);
		assertEquals("1 + 11 * 11 / 11 % 1", s.toString());

		s = new StringBuilder();
		parseNode("${1<1}").appendStructure(s, null);
		parseNode("${1>1}").appendStructure(s, null);
		parseNode("${1<=1}").appendStructure(s, null);
		parseNode("${1>=1}").appendStructure(s, null);
		assertEquals("1 < 11 > 11 <= 11 >= 1", s.toString());

		s = new StringBuilder();
		parseNode("${1==1}").appendStructure(s, null);
		parseNode("${1!=1}").appendStructure(s, null);
		assertEquals("1 == 11 != 1", s.toString());

		s = new StringBuilder();
		parseNode("${1&&1}").appendStructure(s, null);
		parseNode("${1||1}").appendStructure(s, null);
		assertEquals("1 && 11 || 1", s.toString());
	}

	public void testIsLiteralText() {
		assertFalse(parseNode("${1+1}").isLiteralText());
	}

	public void testIsLeftValue() {
		assertFalse(parseNode("${1+1}").isLeftValue());
	}

	public void testGetType() {
		assertNull(parseNode("${1+1}").getType(null, null));
	}

	public void testIsReadOnly() {
		assertTrue(parseNode("${1+1}").isReadOnly(null, null));
	}

	public void testSetValue() {
		try { parseNode("${1+1}").setValue(null, null, null); fail(); } catch (ELException e) {}
	}

	public void testGetValue() {
		assertEquals(Long.valueOf(2l), parseNode("${1+1}").getValue(null, null, null));
		assertEquals("2", parseNode("${1+1}").getValue(null, null, String.class));
	}
}
